import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;

public class ImageCanvas extends JComponent {
		
	private BufferedImage original;
	private BufferedImage scaled;
	private int width;
	private int height;
	private boolean preserveEdges;

	public ImageCanvas() {			
		this.original = null;
		this.preserveEdges = false;
		width = 0;
		height = 0;	
	}
	
	public ImageCanvas(BufferedImage image, boolean preserveEdges) {			
		this.original = image;
		this.preserveEdges = preserveEdges;
		width = 0;
		height = 0;	
	}
	
	public void setImage(BufferedImage image) {
		this.original = image;
		width = 0;
		height = 0;	
	}
			
	public void paint(Graphics g) {
		if (original == null) {
			return;
		}
		int imageHeight = original.getHeight(this);
		int imageWidth = original.getWidth(this);
		float scale = Math.min(this.getWidth() / (float) imageWidth, this.getHeight() / (float) imageHeight);
		int displayWidth = (int) (imageWidth * scale);
		int displayHeight = (int) (imageHeight * scale);
		
		if (displayWidth != width || displayHeight != height) {
			width = displayWidth;
			height = displayHeight;
			if (preserveEdges) {
				scaled = scale(original, width, height);
			}
		}
		
		int dx = (getWidth() - width) / 2; 
		int dy = (getHeight() - height) / 2;
		if (preserveEdges) {
			g.drawImage(
					this.scaled,
					dx, dy, dx + width, dy + height,
					0, 0, width, height,
					Color.WHITE, this);
		} else {
			g.drawImage(
					this.original,
					dx, dy, dx + width, dy + height,
					0, 0, imageWidth, imageHeight,
					Color.WHITE, this);
			
		}
	}
	

	private BufferedImage scale(BufferedImage original, int w, int h) {
		int margin = 2;
		int oldWidth = original.getWidth(this);
		int oldHeight = original.getHeight(this);
		if (oldWidth <=1 || oldHeight <=1) return original;
		if (w < 2*margin || h < 2*margin) return original;
		BufferedImage newImage = new BufferedImage(w, h, BufferedImage.TYPE_3BYTE_BGR);
		Graphics g = newImage.getGraphics();
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, w, h);

		double scale = Math.min((w-2*margin) / (double) oldWidth, (h-2*margin) / (double) oldHeight);

		for (int oldX = 0; oldX < oldWidth; oldX++) {
			int newX = Math.min(Math.max((int) (oldX * scale) + margin, 0), w-1);
			for (int oldY = 0; oldY < oldHeight; oldY++) {
				int newY = Math.min(Math.max((int) (oldY * scale) + margin, 0), h-1);
				int color = original.getRGB(oldX, oldY);
				if (color != -1) { //!white
					newImage.setRGB(newX, newY, color);
				}
			}
		}
		return newImage;
	}

}
